@@ -1,36 +1,31 @@
 import os
 import sys
-from enum import auto
-from unittest import result
 
 import cv2
 import numba
 import numpy as np
-import ome_types.model as om
 import pandas as pd
 import pyqtgraph as pg
 import qdarkstyle
-from sympy import zeros
 import tifffile as tf
 import zarr
-from numpy.lib.type_check import imag
-from ome_types.model.channel import Channel
 from ome_types.model.ome import OME
-from ome_types.model.simple_types import UnitsLength, UnitsTime
-from ome_types.model.tiff_data import TiffData
 from PyQt5.QtCore import *
 from PyQt5.QtGui import QFont, QIcon
 from PyQt5.QtWidgets import *
-from pyqtgraph.colormap import ColorMap
-from scipy.fftpack import fft2, fftshift, ifft2, ifftshift
+from sympy import zeros
 
+from microEye.fitting.gaussian_fit import gaussian_2D_fit
+
+from .fitting.results import *
+from .fitting.fit import *
+
+from .checklist_dialog import Checklist
 from .Filters import *
-from .Fitting import *
 from .metadata import MetadataEditor
 from .Rendering import *
 from .thread_worker import *
 from .uImage import *
-from .checklist_dialog import Checklist
 
 
 class tiff_viewer(QMainWindow):
@@ -176,6 +171,10 @@ class tiff_viewer(QMainWindow):
         self.enableROI.setChecked(False)
         self.enableROI.stateChanged.connect(self.enableROI_changed)
 
+        self.saveCropped = QPushButton(
+            'Save Cropped Image',
+            clicked=lambda: self.save_cropped_img())
+
         self.image_control_layout.addRow(
             self.pages_label,
             self.pages_slider)
@@ -187,6 +186,7 @@ class tiff_viewer(QMainWindow):
             self.max_slider)
         self.image_control_layout.addWidget(self.autostretch)
         self.image_control_layout.addWidget(self.enableROI)
+        self.image_control_layout.addWidget(self.saveCropped)
 
         self.controls_layout.addLayout(
             self.image_control_layout)
@@ -275,10 +275,14 @@ class tiff_viewer(QMainWindow):
 
         # Localization / Render layout
         self.fitting_cbox = QComboBox()
-        self.fitting_cbox.addItem('2D Phasor-Fit')
+        self.fitting_cbox.addItem(
+            '2D Phasor-Fit (CPU)', FittingMethod._2D_Phasor_CPU)
+        self.fitting_cbox.addItem(
+            '2D MLE Gauss-Fit (CPU)', FittingMethod._2D_Gauss_MLE_CPU)
 
         self.render_cbox = QComboBox()
-        self.render_cbox.addItem('2D Gaussian Histogram')
+        self.render_cbox.addItem('2D Histogram', 0)
+        self.render_cbox.addItem('2D Gaussian Histogram', 1)
 
         self.frc_cbox = QComboBox()
         self.frc_cbox.addItem('Binomial')
@@ -288,7 +292,8 @@ class tiff_viewer(QMainWindow):
                 'Exported Columns',
                 ['Frame', 'Coordinates [Pixel]', 'Coordinates [nm]',
                  'Intensity', 'Super-res image', 'Track ID',
-                 'Next NN Distance', 'Number of Merged NNs'], checked=True)
+                 'Next NN Distance', 'Number of Merged NNs',
+                 'X/Y Ratio', 'Sigma X', 'Sigma Y', 'Offset'], checked=True)
 
         self.export_precision = QLineEdit('%10.5f')
 
@@ -448,9 +453,9 @@ class tiff_viewer(QMainWindow):
     def centerROI(self):
         '''Centers the ROI and fits it to the image.
         '''
-        image = self.tiffSeq_Handler[self.pages_slider.value()]
+        image = self.tiffSeq_Handler.getSlice(self.pages_slider.value(), 0, 0)
 
-        self.roi.setSize(image.shape[-2:])
+        self.roi.setSize([image.shape[1], image.shape[0]])
         self.roi.setPos([0, 0])
         self.roi.maxBounds = QRectF(0, 0, image.shape[1], image.shape[0])
 
@@ -517,32 +522,83 @@ class tiff_viewer(QMainWindow):
         else:
             index = self.model.index(i.row(), 0, i.parent())
             self.path = self.model.filePath(index)
+
             if self.tiffSeq_Handler is not None:
                 self.tiffSeq_Handler.close()
-            try:
-                self.tiffSequence = tf.TiffSequence(
-                    self.path + '/' + self.imsq_pattern.text())
-            except ValueError:
-                self.tiffSequence = None
+                self.tiffSeq_Handler = None
 
-            if self.tiffSequence is not None:
-                self.tiffSeq_Handler = TiffSeqHandler(self.tiffSequence)
+            if self.path.endswith('.zarr'):
+                self.tiffSeq_Handler = ZarrImageSequence(self.path)
                 self.tiffSeq_Handler.open()
-                # self.tiffStore = self.tiffSequence.aszarr(axestiled={0: 0})
-                # chunks = (1,) + self.tiffStore._chunks[1:]
-                # self.tiffZar = zarr.open(
-                #     self.tiffStore, mode='r', chunks=chunks)
 
                 self.pages_slider.setMaximum(
-                    self.tiffSeq_Handler.__len__() - 1)
+                        self.tiffSeq_Handler.shape[0] - 1)
                 self.pages_slider.valueChanged.emit(0)
 
-                with tf.TiffFile(self.tiffSequence.files[0]) as fl:
-                    if fl.is_ome:
-                        ome = OME.from_xml(fl.ome_metadata)
-                        self.metadataEditor.pop_OME_XML(ome)
-                        self.px_size.setValue(
-                            self.metadataEditor.px_size.value())
+                self.centerROI()
+            else:
+                try:
+                    self.tiffSequence = tf.TiffSequence(
+                        self.path + '/' + self.imsq_pattern.text())
+                except ValueError:
+                    self.tiffSequence = None
+
+                if self.tiffSequence is not None:
+                    self.tiffSeq_Handler = TiffSeqHandler(self.tiffSequence)
+                    self.tiffSeq_Handler.open()
+
+                    self.pages_slider.setMaximum(
+                        self.tiffSeq_Handler.__len__() - 1)
+                    self.pages_slider.valueChanged.emit(0)
+
+                    with tf.TiffFile(self.tiffSequence.files[0]) as fl:
+                        if fl.is_ome:
+                            ome = OME.from_xml(fl.ome_metadata)
+                            self.metadataEditor.pop_OME_XML(ome)
+                            self.px_size.setValue(
+                                self.metadataEditor.px_size.value())
+
+                    self.centerROI()
+
+    def save_cropped_img(self):
+        if self.tiffSeq_Handler is None:
+            return
+
+        filename, _ = QFileDialog.getSaveFileName(
+            self, "Save Cropped Image",
+            directory=self.path,
+            filter="Zarr Files (*.zarr)")
+
+        roiInfo = self.get_roi_info()
+
+        def work_func():
+            if roiInfo is not None:
+                origin, dim = roiInfo
+
+                ySlice = slice(int(origin[1]), int(origin[1] + dim[1]), 1)
+                xSlice = slice(int(origin[0]), int(origin[0] + dim[0]), 1)
+
+                saveZarrImage(
+                    filename, self.tiffSeq_Handler,
+                    ySlice=ySlice,
+                    xSlice=xSlice
+                )
+            else:
+                origin, dim = None, None
+
+                saveZarrImage(
+                    filename, self.tiffSeq_Handler)
+
+        def done(results):
+            self.saveCropped.setDisabled(False)
+
+        self.worker = thread_worker(
+            work_func,
+            progress=False, z_stage=False)
+        self.worker.signals.result.connect(done)
+        # Execute
+        self.saveCropped.setDisabled(True)
+        self._threadpool.start(self.worker)
 
     def average_stack(self):
         if self.tiffSeq_Handler is not None:
@@ -553,7 +609,7 @@ class tiff_viewer(QMainWindow):
 
     def genOME(self):
         if self.tiffSeq_Handler is not None:
-            frames = len(self.tiffSeq_Handler)
+            frames = self.tiffSeq_Handler.shape[0]
             width = self.image.image.shape[1]
             height = self.image.image.shape[0]
             ome = self.metadataEditor.gen_OME_XML(frames, width, height)
@@ -596,7 +652,8 @@ class tiff_viewer(QMainWindow):
 
     def update_display(self, image=None):
         if image is None:
-            image = self.tiffSeq_Handler[self.pages_slider.value()]
+            image = self.tiffSeq_Handler.getSlice(
+                self.pages_slider.value(), 0, 0)
 
         roiInfo = self.get_roi_info()
         if roiInfo is not None:
@@ -675,7 +732,13 @@ class tiff_viewer(QMainWindow):
                 points[:, 0] += origin[0]
                 points[:, 1] += origin[1]
 
-            sub_fit = phasor_fit(self.uImage._image, points, False)
+            # method
+            method = self.fitting_cbox.currentData()
+
+            if method == FittingMethod._2D_Phasor_CPU:
+                sub_fit = phasor_fit(image, points, False)
+            elif method == FittingMethod._2D_Gauss_MLE_CPU:
+                sub_fit = gaussian_2D_fit(image, points)
 
             if sub_fit is not None:
 
@@ -736,6 +799,7 @@ class tiff_viewer(QMainWindow):
 
     def FRC_estimate(self):
         frc_method = self.frc_cbox.currentText()
+        time = QDateTime.currentDateTime()
         if 'Check' in frc_method:
             img = self.update_loc()
 
@@ -750,6 +814,7 @@ class tiff_viewer(QMainWindow):
 
                 def done(results):
                     plotFRC_(*results)
+                    self.frc_res_btn.setDisabled(False)
             else:
                 return
         elif 'Binomial' in frc_method:
@@ -770,6 +835,7 @@ class tiff_viewer(QMainWindow):
 
                 def done(results):
                     plotFRC(*results)
+                    self.frc_res_btn.setDisabled(False)
             else:
                 return
         else:
@@ -780,6 +846,7 @@ class tiff_viewer(QMainWindow):
             progress=False, z_stage=False)
         self.worker.signals.result.connect(done)
         # Execute
+        self.frc_res_btn.setDisabled(True)
         self._threadpool.start(self.worker)
 
     def drift_cross(self):
@@ -804,12 +871,14 @@ class tiff_viewer(QMainWindow):
                     self.image.setImage(img_norm, autoLevels=False)
                     self.fittingResults = results[0]
                     plot_drift(*results[2])
+                self.drift_cross_btn.setDisabled(False)
 
             self.worker = thread_worker(
                 work_func,
                 progress=False, z_stage=False)
             self.worker.signals.result.connect(done)
             # Execute
+            self.drift_cross_btn.setDisabled(True)
             self._threadpool.start(self.worker)
         else:
             return
@@ -830,12 +899,14 @@ class tiff_viewer(QMainWindow):
                     self.fittingResults = results[0]
                     plot_drift(*results[1])
                     self.update_loc()
+                self.drift_fdm_btn.setDisabled(False)
 
             self.worker = thread_worker(
                 work_func,
                 progress=False, z_stage=False)
             self.worker.signals.result.connect(done)
             # Execute
+            self.drift_fdm_btn.setDisabled(True)
             self._threadpool.start(self.worker)
         else:
             return
@@ -858,12 +929,14 @@ class tiff_viewer(QMainWindow):
 
             def done(results):
                 self.fittingResults = results
+                self.nneigh_merge_btn.setDisabled(False)
 
             self.worker = thread_worker(
                 work_func,
                 progress=False, z_stage=False)
             self.worker.signals.result.connect(done)
             # Execute
+            self.nneigh_merge_btn.setDisabled(True)
             self._threadpool.start(self.worker)
         else:
             return
@@ -885,12 +958,14 @@ class tiff_viewer(QMainWindow):
 
             def done(results):
                 self.fittingResults = results
+                self.nneigh_btn.setDisabled(False)
 
             self.worker = thread_worker(
                 work_func,
                 progress=False, z_stage=False)
             self.worker.signals.result.connect(done)
             # Execute
+            self.nneigh_btn.setDisabled(True)
             self._threadpool.start(self.worker)
         else:
             return
@@ -910,12 +985,14 @@ class tiff_viewer(QMainWindow):
 
             def done(results):
                 self.fittingResults = results
+                self.merge_btn.setDisabled(False)
 
             self.worker = thread_worker(
                 work_func,
                 progress=False, z_stage=False)
             self.worker.signals.result.connect(done)
             # Execute
+            self.merge_btn.setDisabled(True)
             self._threadpool.start(self.worker)
         else:
             return
@@ -924,7 +1001,7 @@ class tiff_viewer(QMainWindow):
         '''Imports fitting results from a (*.tsv) file.
         '''
         filename, _ = QFileDialog.getOpenFileName(
-            self, "Import localizations", filter="TSV Files (*.tsv);;")
+            self, "Import localizations", filter="TSV Files (*.tsv)")
 
         if len(filename) > 0:
 
@@ -952,7 +1029,7 @@ class tiff_viewer(QMainWindow):
 
         if filename is None:
             filename, _ = QFileDialog.getSaveFileName(
-                self, "Export localizations", filter="TSV Files (*.tsv);;")
+                self, "Export localizations", filter="TSV Files (*.tsv)")
 
         if len(filename) > 0:
             options = self.export_options.toList()
@@ -963,16 +1040,21 @@ class tiff_viewer(QMainWindow):
                 'Coordinates [Pixel]' in options,
                 'Coordinates [nm]' in options,
                 'Coordinates [nm]' in options,
+                'Coordinates [nm]' in options,
                 'Intensity' in options,
                 'Track ID' in options,
                 'Next NN Distance' in options,
-                'Number of Merged NNs' in options
+                'Number of Merged NNs' in options,
+                'X/Y Ratio' in options,
+                'Sigma X' in options,
+                'Sigma Y' in options,
+                'Offset' in options
             ]
 
             dataFrame = self.fittingResults.dataFrame()
             dataFrame.to_csv(
                 filename, index=False,
-                columns=FittingResults.columns[exp_columns],
+                columns=DataColumns.values()[exp_columns],
                 float_format=self.export_precision.text(),
                 sep='\t',
                 encoding='utf-8')
@@ -994,15 +1076,19 @@ class tiff_viewer(QMainWindow):
             return
 
         filename, _ = QFileDialog.getSaveFileName(
-            self, "Save localizations", filter="TSV Files (*.tsv);;")
+            self, "Save localizations", filter="TSV Files (*.tsv)")
 
         if len(filename) > 0:
+            def done(res):
+                self.loc_btn.setDisabled(False)
             # Any other args, kwargs are passed to the run function
             self.worker = thread_worker(
                 self.proccess_loc, filename,
                 progress=True, z_stage=False)
             self.worker.signals.progress.connect(self.update_loc)
+            self.worker.signals.result.connect(done)
             # Execute
+            self.loc_btn.setDisabled(True)
             self._threadpool.start(self.worker)
 
     def update_loc(self):
@@ -1016,7 +1102,11 @@ class tiff_viewer(QMainWindow):
         if self.fittingResults is None:
             return None
         elif len(self.fittingResults) > 0:
-            renderClass = gauss_hist_render(self.super_px_size.value())
+            render_idx = self.render_cbox.currentData()
+            if render_idx == 0:
+                renderClass = hist2D_render(self.super_px_size.value())
+            elif render_idx == 1:
+                renderClass = gauss_hist_render(self.super_px_size.value())
             img = renderClass.render(
                 *self.fittingResults.toRender())
             img_norm = cv2.normalize(
@@ -1057,7 +1147,8 @@ class tiff_viewer(QMainWindow):
         )
 
         self.thread_done = 0  # number of threads done
-        time = QDateTime.currentDateTime()  # timer
+        start = QDateTime.currentDateTime()  # timer
+        time = start
 
         # Filters + Blob detector params
         filter = self.bandpassFilter.filter
@@ -1065,6 +1156,9 @@ class tiff_viewer(QMainWindow):
         tempEnabled = self.tempMedianFilter.enabled.isChecked()
         params = self.get_blob_detector_params()
 
+        # method
+        method = self.fitting_cbox.currentData()
+
         # ROI
         roiInfo = self.get_roi_info()
         self.enableROI.setChecked(False)
@@ -1073,14 +1167,19 @@ class tiff_viewer(QMainWindow):
         threads = self._threadpool.maxThreadCount() - 2
         print('Threads', threads)
         for i in range(
-                0, int(np.ceil(len(self.tiffSeq_Handler) / threads) * threads),
+                0, int(
+                    np.ceil(
+                        self.tiffSeq_Handler.shape[0] / threads
+                        ) * threads
+                        ),
                 threads):
             time = QDateTime.currentDateTime()
             workers = []
             self.thread_done = 0
             for k in range(threads):
                 if i + k < len(self.tiffSeq_Handler):
-                    img = self.tiffSeq_Handler[i + k].copy()
+                    img = self.tiffSeq_Handler.getSlice(
+                        i + k, 0, 0)
 
                     temp = None
                     if tempEnabled:
@@ -1096,6 +1195,7 @@ class tiff_viewer(QMainWindow):
                         filter,
                         params,
                         roiInfo,
+                        method,
                         progress=False, z_stage=False)
                     worker.signals.result.connect(self.update_lists)
                     workers.append(worker)
@@ -1105,10 +1205,11 @@ class tiff_viewer(QMainWindow):
                 QThread.msleep(10)
 
             exex = time.msecsTo(QDateTime.currentDateTime())
+            duration = start.msecsTo(QDateTime.currentDateTime())
 
             print(
                 'index: {:d}/{:d}, Time: {:d}  '.format(
-                    i + len(workers), len(self.tiffSeq_Handler), exex),
+                    i + len(workers), self.tiffSeq_Handler.shape[0], exex),
                 end="\r")
             if (i // threads) % 40 == 0:
                 progress_callback.emit(self)
@@ -1122,12 +1223,17 @@ class tiff_viewer(QMainWindow):
             temp: TemporalMedianFilter,
             filter: AbstractFilter,
             params: cv2.SimpleBlobDetector_Params,
-            roiInfo):
+            roiInfo,
+            method=FittingMethod._2D_Phasor_CPU):
+
+        filtered = image.copy()
 
         # apply the median filter
         if temp is not None:
             frames = temp.getFrames(index, self.tiffSeq_Handler)
-            image = temp.run(image, frames, roiInfo)
+            filtered = temp.run(image, frames, roiInfo)
+        # else:
+        #     filtered_img = None
 
         # crop image to ROI
         if roiInfo is not None:
@@ -1136,18 +1242,29 @@ class tiff_viewer(QMainWindow):
             image = image[
                 int(origin[1]):int(origin[1] + dim[1]),
                 int(origin[0]):int(origin[0] + dim[0])]
+            filtered = filtered[
+                int(origin[1]):int(origin[1] + dim[1]),
+                int(origin[0]):int(origin[0] + dim[0])]
+
+            # if filtered_img is not None:
+            #     filtered_img = filtered_img[
+            #         int(origin[1]):int(origin[1] + dim[1]),
+            #         int(origin[0]):int(origin[0] + dim[0])]
 
         results = localize_frame(
             index=index,
             image=image,
+            filtered=filtered,
             filter=filter,
             params=params,
-            threshold=self.th_min_slider.value()
+            threshold=self.th_min_slider.value(),
+            method=method
         )
 
-        if len(results) > 0 and roiInfo is not None:
-            results[:, 0] += origin[0]
-            results[:, 1] += origin[1]
+        if results is not None:
+            if len(results) > 0 and roiInfo is not None:
+                results[:, 0] += origin[0]
+                results[:, 1] += origin[1]
 
         return results
 
