@@ -6,7 +6,6 @@ import traceback
 from queue import Queue
 
 import cv2
-from lmfit.minimizer import minimize
 import numpy as np
 import pyqtgraph as pg
 import tifffile as tf
@@ -17,8 +16,6 @@ from PyQt5.QtWidgets import *
 from pyqtgraph import PlotWidget, plot
 from pyqtgraph.metaarray.MetaArray import axis
 
-from tifffile.tifffile import astype
-
 from .camera_calibration import dark_calibration
 
 from ..qlist_slider import *
@@ -1055,7 +1052,6 @@ class IDS_Panel(QGroupBox):
 
                     # get frame and temp to save from bottom of stack
                     frame, temp = self._frames.get()
-                    frame = frame.copy()
 
                     # creates dir
                     if tempFile is None:
@@ -1125,7 +1121,8 @@ class IDS_Panel(QGroupBox):
                 if darkCal._counter > 1:
                     darkCal.saveResults(path)
 
-            saveMetadata(index, frames_saved)
+            if self.cam_save_temp.isChecked():
+                saveMetadata(index)
 
     @staticmethod
     def find_nearest(array, value):
