@@ -17,6 +17,8 @@ from pyqtgraph import PlotWidget, plot
 from pyqtgraph.metaarray.MetaArray import axis
 from tifffile.tifffile import astype
 
+from .camera_calibration import dark_calibration
+
 from ..metadata import MetadataEditor
 from ..qlist_slider import *
 from ..thread_worker import *
@@ -73,17 +75,18 @@ class Vimba_Panel(QGroupBox):
         self._buffer = Queue()  # data memory stack
         self._temps = Queue()  # camera sensor temperature stack
         # stack of (frame, temperature) tuples (for saving)
+        self.frame = None
         self._frames = Queue()
         # reserved for testing/displaying execution time
         self._exec_time = 0
         # reserved for testing/displaying execution time
         self._save_time = 0
-        # reserved for testing/displaying execution time
-        self._dis_time = 0
-        self._zoom = 0.50  # display resize
+        # reserved for testing/displaying execution tim
         self._directory = ""  # save directory
         self._save_path = ""  # save path
 
+        self._dis_time = 0
+
         self._counter = 0
         self._nFrames = 1
 
@@ -103,6 +106,7 @@ class Vimba_Panel(QGroupBox):
         self.first_tab = QWidget()
         self.second_tab = QWidget()
         self.third_tab = QWidget()
+        self.fourth_tab = QWidget()
 
         self.OME_tab = MetadataEditor()
         self.OME_tab.channel_name.setText(self._cam.name)
@@ -115,24 +119,30 @@ class Vimba_Panel(QGroupBox):
         self.main_tab_view.addTab(self.first_tab, "Main")
         self.main_tab_view.addTab(self.second_tab, "Preview")
         self.main_tab_view.addTab(self.third_tab, "Area of Interest (AOI)")
+        self.main_tab_view.addTab(self.fourth_tab, "GPIOs")
         if not self.mini:
             self.main_tab_view.addTab(self.OME_tab, "OME-XML metadata")
 
         # first tab vertical layout
-        self.first_tab_Layout = QVBoxLayout()
+        self.first_tab_Layout = QFormLayout()
         # set as first tab layout
         self.first_tab.setLayout(self.first_tab_Layout)
 
         # second tab vertical layout
-        self.second_tab_Layout = QVBoxLayout()
+        self.second_tab_Layout = QFormLayout()
         # set as second tab layout
         self.second_tab.setLayout(self.second_tab_Layout)
 
         # third tab vertical layout
-        self.third_tab_Layout = QVBoxLayout()
+        self.third_tab_Layout = QFormLayout()
         # set as third tab layout
         self.third_tab.setLayout(self.third_tab_Layout)
 
+        # fourth tab vertical layout
+        self.fourth_tab_Layout = QFormLayout()
+        # set as fourth tab layout
+        self.fourth_tab.setLayout(self.fourth_tab_Layout)
+
         # exposure slider control
         self.cam_exposure_lbl = DragLabel(
             "Exposure " + self.cam.exposure_unit,
@@ -155,6 +165,40 @@ class Vimba_Panel(QGroupBox):
         self.cam_exposure_qs.setSuffix(self._cam.exposure_unit)
         self.cam_exposure_qs.valueChanged.connect(self.exposure_spin_changed)
 
+        self.cam_exp_shortcuts = QHBoxLayout()
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton(
+                'min',
+                clicked=lambda: self.cam_exposure_qs.setValue(self._cam.exposure_range[0]))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('1ms', clicked=lambda: self.cam_exposure_qs.setValue(1e3))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('10ms', clicked=lambda: self.cam_exposure_qs.setValue(1e4))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('30ms', clicked=lambda: self.cam_exposure_qs.setValue(3e4))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('50ms', clicked=lambda: self.cam_exposure_qs.setValue(5e4))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('100ms', clicked=lambda: self.cam_exposure_qs.setValue(1e5))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('150ms', clicked=lambda: self.cam_exposure_qs.setValue(1.5e5))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('200ms', clicked=lambda: self.cam_exposure_qs.setValue(2e5))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('500ms', clicked=lambda: self.cam_exposure_qs.setValue(5e5))
+        )
+        self.cam_exp_shortcuts.addWidget(
+            QPushButton('1s', clicked=lambda: self.cam_exposure_qs.setValue(1e6))
+        )
+
         # exposure mode combobox
         self.cam_exposure_mode_lbl = QLabel("Exposure Mode")
         self.cam_exposure_mode_cbox = QComboBox()
@@ -278,6 +322,9 @@ class Vimba_Panel(QGroupBox):
         # save to hard drive checkbox
         self.cam_save_temp = QCheckBox("Save to dir")
         self.cam_save_temp.setChecked(self.mini)
+        self.dark_cal = QCheckBox("Dark Calibration")
+        self.dark_cal.setToolTip('Generates mean and variance images')
+        self.dark_cal.setChecked(False)
         self.save_bigg_tiff = QCheckBox("BiggTiff Format")
         self.save_bigg_tiff.setChecked(True)
         self.cam_save_meta = QCheckBox("Write full OME-XML")
@@ -296,19 +343,13 @@ class Vimba_Panel(QGroupBox):
         self.lut_btns.addButton(self.fast_lut_rbtn)
 
         # display size
-        self.zoom_layout = QHBoxLayout()
-        self.zoom_lbl = QLabel("Resize " + "{:.0f}%".format(self._zoom*100))
-        self.zoom_in_btn = QPushButton(
-            "+",
-            clicked=lambda: self.zoom_in()
-        )
-        self.zoom_out_btn = QPushButton(
-            "-",
-            clicked=lambda: self.zoom_out()
-        )
-        self.zoom_layout.addWidget(self.zoom_lbl, 4)
-        self.zoom_layout.addWidget(self.zoom_out_btn, 1)
-        self.zoom_layout.addWidget(self.zoom_in_btn, 1)
+        self.zoom_lbl = QLabel("Resize Display:")
+        self.zoom_box = QDoubleSpinBox()
+        self.zoom_box.setSingleStep(0.02)
+        self.zoom_box.setMinimum(0.1)
+        self.zoom_box.setMaximum(4.0)
+        self.zoom_box.setDecimals(2)
+        self.zoom_box.setValue(0.5)
 
         # controls for histogram stretching and plot
         self.histogram_lbl = QLabel("Histogram")
@@ -365,73 +406,126 @@ class Vimba_Panel(QGroupBox):
             "Center AOI",
             clicked=lambda: self.center_AOI()
         )
+        self.AOI_select_btn = QPushButton(
+            "Select AOI",
+            clicked=lambda: self.select_AOI()
+        )
+
+        # GPIOs
+        self.lineSelector = QComboBox()
+        self.lineMode = QComboBox()
+        self.lineSource = QComboBox()
+        self.lineInverter = QCheckBox('Inverter')
+        self.lineInverter.setChecked(False)
+
+        with self._cam.cam:
+            self.lineSelector.addItems(self.cam.get_io_lines())
+            self.lineMode.addItems(self.cam.get_line_modes())
+            self.lineSource.addItems(self.cam.get_line_sources())
+
+        self.lineSelector.currentIndexChanged.connect(self.io_line_changed)
+
+        self.set_io_config = QPushButton(
+            'Set Config.',
+            clicked=lambda: self.set_io_line_config())
 
         # adding widgets to the main layout
-        self.first_tab_Layout.addWidget(self.cam_trigger_mode_lbl)
-        self.first_tab_Layout.addWidget(self.cam_trigger_mode_cbox)
-        self.first_tab_Layout.addWidget(self.cam_trigger_source_lbl)
-        self.first_tab_Layout.addWidget(self.cam_trigger_source_cbox)
-        self.first_tab_Layout.addWidget(self.cam_trigger_selector_lbl)
-        self.first_tab_Layout.addWidget(self.cam_trigger_selector_cbox)
-        self.first_tab_Layout.addWidget(self.cam_trigger_activation_lbl)
-        self.first_tab_Layout.addWidget(self.cam_trigger_activation_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_trigger_mode_lbl,
+            self.cam_trigger_mode_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_trigger_source_lbl,
+            self.cam_trigger_source_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_trigger_selector_lbl,
+            self.cam_trigger_selector_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_trigger_activation_lbl,
+            self.cam_trigger_activation_cbox)
 
-        self.first_tab_Layout.addWidget(self.cam_exposure_lbl)
-        self.first_tab_Layout.addWidget(self.cam_exposure_qs)
-        self.first_tab_Layout.addWidget(self.cam_exposure_slider)
-        self.first_tab_Layout.addWidget(self.cam_exposure_mode_lbl)
-        self.first_tab_Layout.addWidget(self.cam_exposure_mode_cbox)
-        self.first_tab_Layout.addWidget(self.cam_exposure_auto_lbl)
-        self.first_tab_Layout.addWidget(self.cam_exposure_auto_cbox)
-        self.first_tab_Layout.addWidget(self.cam_pixel_format_lbl)
-        self.first_tab_Layout.addWidget(self.cam_pixel_format_cbox)
-        self.first_tab_Layout.addLayout(self.config_Hlay)
-        self.first_tab_Layout.addWidget(self.cam_freerun_btn)
-        self.first_tab_Layout.addWidget(self.cam_trigger_btn)
-        self.first_tab_Layout.addWidget(self.cam_stop_btn)
+        self.first_tab_Layout.addRow(
+            self.cam_exposure_lbl,
+            self.cam_exposure_qs)
+        self.first_tab_Layout.addRow(self.cam_exposure_slider)
+        self.first_tab_Layout.addRow(self.cam_exp_shortcuts)
+        self.first_tab_Layout.addRow(
+            self.cam_exposure_mode_lbl,
+            self.cam_exposure_mode_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_exposure_auto_lbl,
+            self.cam_exposure_auto_cbox)
+        self.first_tab_Layout.addRow(
+            self.cam_pixel_format_lbl,
+            self.cam_pixel_format_cbox)
+        self.first_tab_Layout.addRow(self.config_Hlay)
+        self.first_tab_Layout.addRow(self.cam_freerun_btn)
+        self.first_tab_Layout.addRow(self.cam_trigger_btn)
+        self.first_tab_Layout.addRow(self.cam_stop_btn)
         if not self.mini:
-            self.first_tab_Layout.addWidget(QLabel("Experiment:"))
-            self.first_tab_Layout.addWidget(self.experiment_name)
-            self.first_tab_Layout.addWidget(QLabel("Save Directory:"))
-            self.first_tab_Layout.addLayout(self.save_dir_layout)
-            self.first_tab_Layout.addWidget(self.frames_tbox)
+            self.first_tab_Layout.addRow(
+                QLabel("Experiment:"),
+                self.experiment_name)
+            self.first_tab_Layout.addRow(
+                QLabel("Save Directory:"),
+                self.save_dir_layout)
+            self.first_tab_Layout.addRow(
+                QLabel('Frames:'),
+                self.frames_tbox)
             self.first_tab_Layout.addWidget(self.cam_save_temp)
+            self.first_tab_Layout.addWidget(self.dark_cal)
             self.first_tab_Layout.addWidget(self.save_bigg_tiff)
             self.first_tab_Layout.addWidget(self.cam_save_meta)
-        self.first_tab_Layout.addStretch()
 
         if not self.mini:
-            self.second_tab_Layout.addWidget(self.preview_ch_box)
-        self.second_tab_Layout.addWidget(self.slow_lut_rbtn)
-        self.second_tab_Layout.addWidget(self.fast_lut_rbtn)
+            self.second_tab_Layout.addRow(self.preview_ch_box)
+        self.second_tab_Layout.addRow(self.slow_lut_rbtn)
+        self.second_tab_Layout.addRow(self.fast_lut_rbtn)
         if not self.mini:
-            self.second_tab_Layout.addLayout(self.zoom_layout)
+            self.second_tab_Layout.addRow(
+                self.zoom_lbl,
+                self.zoom_box)
 
-        self.second_tab_Layout.addWidget(self.histogram_lbl)
-        self.second_tab_Layout.addWidget(self.alpha)
+        self.second_tab_Layout.addRow(
+            self.histogram_lbl,
+            self.alpha)
         self.second_tab_Layout.addWidget(self.beta)
         self.second_tab_Layout.addWidget(self.auto_stretch)
-        self.second_tab_Layout.addWidget(self.histogram)
-        self.second_tab_Layout.addWidget(self.hist_cdf)
-        self.second_tab_Layout.addWidget(QLabel("Stats"))
-        self.second_tab_Layout.addWidget(self.info_temp)
+        self.second_tab_Layout.addRow(self.histogram)
+        self.second_tab_Layout.addRow(self.hist_cdf)
+        self.second_tab_Layout.addRow(
+            QLabel("Stats:"), self.info_temp)
         self.second_tab_Layout.addWidget(self.info_cap)
         self.second_tab_Layout.addWidget(self.info_disp)
         self.second_tab_Layout.addWidget(self.info_save)
-        self.second_tab_Layout.addStretch()
 
-        self.third_tab_Layout.addWidget(QLabel("X"))
-        self.third_tab_Layout.addWidget(self.AOI_x_tbox)
-        self.third_tab_Layout.addWidget(QLabel("Y"))
-        self.third_tab_Layout.addWidget(self.AOI_y_tbox)
-        self.third_tab_Layout.addWidget(QLabel("Width"))
-        self.third_tab_Layout.addWidget(self.AOI_width_tbox)
-        self.third_tab_Layout.addWidget(QLabel("Height"))
-        self.third_tab_Layout.addWidget(self.AOI_height_tbox)
-        self.third_tab_Layout.addWidget(self.AOI_set_btn)
-        self.third_tab_Layout.addWidget(self.AOI_reset_btn)
-        self.third_tab_Layout.addWidget(self.AOI_center_btn)
-        self.third_tab_Layout.addStretch()
+        self.third_tab_Layout.addRow(
+            QLabel("X:"),
+            self.AOI_x_tbox)
+        self.third_tab_Layout.addRow(
+            QLabel("Y:"),
+            self.AOI_y_tbox)
+        self.third_tab_Layout.addRow(
+            QLabel("Width:"),
+            self.AOI_width_tbox)
+        self.third_tab_Layout.addRow(
+            QLabel("Height:"),
+            self.AOI_height_tbox)
+        self.third_tab_Layout.addRow(self.AOI_set_btn)
+        self.third_tab_Layout.addRow(self.AOI_reset_btn)
+        self.third_tab_Layout.addRow(self.AOI_center_btn)
+        self.third_tab_Layout.addRow(self.AOI_select_btn)
+
+        self.fourth_tab_Layout.addRow(
+            QLabel('Selected Line:'),
+            self.lineSelector)
+        self.fourth_tab_Layout.addRow(
+            QLabel('Line Mode:'),
+            self.lineMode)
+        self.fourth_tab_Layout.addRow(
+            QLabel('Line Source:'),
+            self.lineSource)
+        self.fourth_tab_Layout.addWidget(
+            self.set_io_config)
 
     @property
     def cam(self):
@@ -489,10 +583,10 @@ class Vimba_Panel(QGroupBox):
                 int(self.AOI_x_tbox.text()),
                 int(self.AOI_y_tbox.text()))
 
-        self.AOI_x_tbox.setText(str(self.cam.offsetX))
-        self.AOI_y_tbox.setText(str(self.cam.offsetY))
-        self.AOI_width_tbox.setText(str(self.cam.width))
-        self.AOI_height_tbox.setText(str(self.cam.height))
+        self.AOI_x_tbox.setText(str(int(self.cam.offsetX)))
+        self.AOI_y_tbox.setText(str(int(self.cam.offsetY)))
+        self.AOI_width_tbox.setText(str(int(self.cam.width)))
+        self.AOI_height_tbox.setText(str(int(self.cam.height)))
 
     def reset_AOI(self):
         '''Resets the AOI for the slected IDS_Camera
@@ -506,8 +600,8 @@ class Vimba_Panel(QGroupBox):
             self.cam.set_roi(self.cam.width_max, self.cam.height_max)
         self.AOI_x_tbox.setText("0")
         self.AOI_y_tbox.setText("0")
-        self.AOI_width_tbox.setText(str(self.cam.width))
-        self.AOI_height_tbox.setText(str(self.cam.height))
+        self.AOI_width_tbox.setText(str(int(self.cam.width)))
+        self.AOI_height_tbox.setText(str(int(self.cam.height)))
 
     def center_AOI(self):
         '''Sets the AOI for the slected vimba_cam
@@ -527,15 +621,16 @@ class Vimba_Panel(QGroupBox):
         self.AOI_width_tbox.setText(str(self.cam.width))
         self.AOI_height_tbox.setText(str(self.cam.height))
 
-    def zoom_in(self):
-        """Increase image display size"""
-        self._zoom = min(self._zoom + 0.05, 4)
-        self.zoom_lbl.setText("Resize " + "{:.0f}%".format(self._zoom*100))
+    def select_AOI(self):
+        if self.frame is not None:
+            aoi = cv2.selectROI(self.frame._view)
+            cv2.destroyWindow('ROI selector')
 
-    def zoom_out(self):
-        """Decrease image display size"""
-        self._zoom = max(self._zoom - 0.05, 0.25)
-        self.zoom_lbl.setText("Resize " + "{:.0f}%".format(self._zoom*100))
+            z = self.zoom_box.value()
+            self.AOI_x_tbox.setText(str(int(aoi[0] / z)))
+            self.AOI_y_tbox.setText(str(int(aoi[1] / z)))
+            self.AOI_width_tbox.setText(str(int(aoi[2] / z)))
+            self.AOI_height_tbox.setText(str(int(aoi[3] / z)))
 
     @pyqtSlot()
     def save_browse_clicked(self):
@@ -596,11 +691,7 @@ class Vimba_Panel(QGroupBox):
             self._cam.set_exposure(value)
             self._cam.get_exposure(False)
 
-        self.cam_exposure_qs.valueChanged.disconnect(
-            self.exposure_spin_changed)
-        self.cam_exposure_qs.setValue(self._cam.exposure_current)
-        self.cam_exposure_qs.valueChanged.connect(
-            self.exposure_spin_changed)
+        self.refresh_exposure()
 
         self.OME_tab.exposure.setValue(self._cam.exposure_current / 1000)
         if self.master:
@@ -620,16 +711,54 @@ class Vimba_Panel(QGroupBox):
             self._cam.set_exposure(value)
             self._cam.get_exposure(False)
 
-        self.cam_exposure_slider.elementChanged[int, float] \
-            .connect(self.cam_exposure_value_changed)
-        self.cam_exposure_slider.setNearest(self._cam.exposure_current)
-        self.cam_exposure_slider.elementChanged[int, float] \
-            .disconnect(self.cam_exposure_value_changed)
+        self.refresh_exposure()
 
-        self.OME_tab.exposure.setText(str(self._cam.exposure_current / 1000))
+        self.OME_tab.exposure.setValue(self._cam.exposure_current / 1000)
         if self.master:
             self.exposureChanged.emit()
 
+    def refresh_exposure(self):
+
+        self.cam_exposure_slider.elementChanged[int, float] \
+            .disconnect(self.cam_exposure_value_changed)
+        self.cam_exposure_slider.setNearest(self._cam.exposure_current)
+        self.cam_exposure_slider.elementChanged[int, float] \
+            .connect(self.cam_exposure_value_changed)
+        
+        self.cam_exposure_qs.valueChanged.disconnect(
+            self.exposure_spin_changed)
+        self.cam_exposure_qs.setValue(self._cam.exposure_current)
+        self.cam_exposure_qs.valueChanged.connect(
+            self.exposure_spin_changed)
+
+    def io_line_changed(self, value):
+        with self._cam.cam:
+            line = self.lineSelector.currentText()
+            self.cam.select_io_line(line)
+            mode = self.cam.get_line_mode()
+            self.lineMode.setCurrentText(
+                mode)
+            if 'Out' in mode:
+                self.lineSource.setCurrentText(
+                    self.cam.get_line_source()
+                )
+                self.lineInverter.setChecked(
+                    self.cam.get_line_inverter()
+                )
+
+    def set_io_line_config(self):
+        with self._cam.cam:
+            line = self.lineSelector.currentText()
+            self.cam.select_io_line(line)
+            mode = self.lineMode.currentText()
+            self.cam.set_line_mode(
+                mode)
+            if 'Out' in mode:
+                self.cam.set_line_source(
+                    self.lineSource.currentText())
+                self.cam.set_line_inverter(
+                    self.lineInverter.isChecked())
+
     def start_free_run(self, cam: vimba_cam):
         """
         Starts free run acquisition mode
@@ -734,7 +863,7 @@ class Vimba_Panel(QGroupBox):
                 # Execute
                 self._threadpool.start(self.s_worker)
 
-    def _capture_handler(self, cam: vb.Camera, frame):
+    def _capture_handler(self, cam, frame):
         self._buffer.put(frame.as_numpy_ndarray())
         cam.queue_frame(frame)
         # add sensor temperature to the stack
@@ -803,7 +932,7 @@ class Vimba_Panel(QGroupBox):
 
                     # reshape image into proper shape
                     # (height, width, bytes per pixel)
-                    frame = uImage(self._buffer.get()) \
+                    self.frame = uImage(self._buffer.get()) \
                         if self.cam.bytes_per_pixel > 1\
                         else uImage.fromUINT8(
                             self._buffer.get(),
@@ -812,9 +941,9 @@ class Vimba_Panel(QGroupBox):
                     # add to saving stack
                     if self.cam_save_temp.isChecked():
                         if not self.mini:
-                            self._frames.put((frame.image, self._temps.get()))
+                            self._frames.put((self.frame.image, self._temps.get()))
                         else:
-                            self._frames.put(frame.image)
+                            self._frames.put(self.frame.image)
 
                     if self.preview_ch_box.isChecked():
                         _range = None
@@ -822,23 +951,25 @@ class Vimba_Panel(QGroupBox):
                         if not self.auto_stretch.isChecked():
                             _range = (self.alpha.value(), self.beta.value())
 
-                        frame.equalizeLUT(
+                        self.frame.equalizeLUT(
                             _range, self.slow_lut_rbtn.isChecked())
                         if self.auto_stretch.isChecked():
-                            self.alpha.setValue(frame._min)
-                            self.beta.setValue(frame._max)
-                        self.histogram.setXRange(frame._min, frame._max)
-                        self.hist_cdf.setXRange(frame._min, frame._max)
+                            self.alpha.setValue(self.frame._min)
+                            self.beta.setValue(self.frame._max)
+                        self.histogram.setXRange(self.frame._min, self.frame._max)
+                        self.hist_cdf.setXRange(self.frame._min, self.frame._max)
 
-                        self._plot_ref.setData(frame._hist[:, 0])
-                        self._cdf_plot_ref.setData(frame._cdf)
+                        self._plot_ref.setData(self.frame._hist[:, 0])
+                        self._cdf_plot_ref.setData(self.frame._cdf)
 
                         # resizing the image
-                        frame._view = cv2.resize(
-                            frame._view, (0, 0), fx=self._zoom, fy=self._zoom)
+                        self.frame._view = cv2.resize(
+                            self.frame._view, (0, 0),
+                            fx=self.zoom_box.value(),
+                            fy=self.zoom_box.value())
 
                         # display it
-                        cv2.imshow(cam.name, frame._view)
+                        cv2.imshow(cam.name, self.frame._view)
                         cv2.waitKey(1)
                     else:
                         cv2.waitKey(1)
@@ -872,6 +1003,7 @@ class Vimba_Panel(QGroupBox):
             path = self._save_path
             tiffWriter = None
             tempFile = None
+            darkCal = None
             index = 0
             biggTiff = self.save_bigg_tiff.isChecked()
 
@@ -903,6 +1035,9 @@ class Vimba_Panel(QGroupBox):
                     # for save time estimations
                     time = QDateTime.currentDateTime()
 
+                    # get frame and temp to save from bottom of stack
+                    frame, temp = self._frames.get()
+
                     if tempFile is None:
                         if not os.path.exists(path):
                             os.makedirs(path)
@@ -916,11 +1051,12 @@ class Vimba_Panel(QGroupBox):
                             bigtiff=biggTiff,
                             ome=False)
 
-                    # get frame and temp to save from bottom of stack
-                    frame, temp = self._frames.get()
-                    frame = frame.copy()
+                    if self.dark_cal.isChecked():
+                        if darkCal is None:
+                            exp = self._cam.exposure_current
+                            darkCal = dark_calibration(frame.shape, exp)
 
-                    # creates dir
+                        darkCal.addFrame(frame)
 
                     # append frame to tiff
                     try:
@@ -965,8 +1101,12 @@ class Vimba_Panel(QGroupBox):
                 tempFile.close()
             if tiffWriter is not None:
                 tiffWriter.close()
+            if darkCal is not None:
+                if darkCal._counter > 1:
+                    darkCal.saveResults(path)
 
-            saveMetadata(index, frames_saved)
+            if self.cam_save_temp.isChecked():
+                saveMetadata(index)
 
     @staticmethod
     def find_nearest(array, value):
@@ -1011,10 +1151,10 @@ class Vimba_Panel(QGroupBox):
 
                 w, h, x, y = self.cam.get_roi()
 
-                self.AOI_x_tbox.setText(str(x))
-                self.AOI_y_tbox.setText(str(y))
-                self.AOI_width_tbox.setText(str(w))
-                self.AOI_height_tbox.setText(str(h))
+                self.AOI_x_tbox.setText(str(int(x)))
+                self.AOI_y_tbox.setText(str(int(y)))
+                self.AOI_width_tbox.setText(str(int(w)))
+                self.AOI_height_tbox.setText(str(int(h)))
 
                 self.cam_trigger_mode_cbox.setCurrentText(
                     self.cam.get_trigger_mode())
